/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.cat.glossary.engine.regex.ui;

import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtension;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Plugin;
import org.eclipse.jface.preference.IPreferenceStore;
import org.osgi.framework.BundleContext;

import benten.cat.glossary.core.BentenGlossaryDriver;
import benten.cat.glossary.core.BentenGlossaryDriverManager;
import benten.cat.glossary.core.BentenGlossaryEngine;
import benten.cat.glossary.data.AbstractBentenGlossaryDataTsvPlugin;
import benten.cat.glossary.engine.AbstractBentenGlossaryEnginePlugin;
import benten.cat.glossary.engine.regex.RegexGlossaryDriver;
import benten.cat.glossary.engine.regex.ui.messages.RegexGlossaryPluginMessages;
import benten.ui.preference.BentenPreference;
import benten.ui.preference.BentenPreference.Preference;

/**
 * 正規表現用語集ドライバー
 *
 * このクラスは、TM エンジンの切り替えテストのためのテストクラスです。
 * 実際の Benten 基本セットには含まれません。
 * 
 * <UL>
 * <LI>このプラグインを Eclipse 上で有効化するためのプラグイン・クラスです。
 * </UL>
 * 
 * @author IGA Tosiki
 */
public class RegexGlossaryPlugin extends AbstractBentenGlossaryEnginePlugin {
	/**
	 * Benten 正規表現 (Regex) 用語集エンジン・プラグインのためのメッセージ。
	 */
	protected static final RegexGlossaryPluginMessages fMsg = new RegexGlossaryPluginMessages();

	/**
	 * デフォルト用語集の名称。
	 */
	private static final String DEFAULT_GLOSSARY = "Default Glossary"; //$NON-NLS-1$

	@Override
	public void start(final BundleContext context) throws Exception {
		super.start(context);
		plugin = this;
	}

	@Override
	public void stop(final BundleContext context) throws Exception {
		plugin = null;
		super.stop(context);
	}

	@Override
	public BentenGlossaryDriver getDriver() {
		try {
			// 自らのドライバーを自己登録します。
			Class.forName("benten.cat.glossary.engine.regex.RegexGlossaryDriver"); //$NON-NLS-1$
		} catch (final ClassNotFoundException e) {
			getDefault().log(fMsg.getGetDriverClassNotFoundException());
			getDefault().log(e);
		}

		return BentenGlossaryDriverManager.getDriver(RegexGlossaryDriver.DRIVER_NAME);
	}

	@Override
	public BentenGlossaryEngine getEngine() {
		if (fEngine == null) {
			fEngine = getDriver().getEngineInstance();
		}

		return fEngine;
	}

	@Override
	public void load() {
		getDefault().log(fMsg.getLoadI001());

		getEngine().unload();

		final File tsvDir = getTsvDir();
		try {
			getEngine().loadTsv(tsvDir);
		} catch (final IOException e) {
			getDefault().log(e);
		}
	}

	/**
	 * 拡張ポイントなどから TSV ディレクトリーを取得します。
	 * @return TSV ディレクトリー。必ず null 以外が戻ります。
	 */
	@SuppressWarnings("static-access")
	private File getTsvDir() {
		File result = null;

		final IPreferenceStore store = BentenPreference.getStore();
		final boolean useTsvLocation = store.getBoolean(Preference.USE_TSV_LOCATION.name());
		if (useTsvLocation) {
			final String tsvFolder = store.getString(Preference.TSV_LOCATION.name());
			result = new File(tsvFolder);
		} else {
			result = getTsvDataPlugin().getDefault().getFile("/glossary"); //$NON-NLS-1$
		}
		getDefault().log(fMsg.getGetTsvDirI001(result.toString()));

		if (result.exists() == false) {
			throw new IllegalArgumentException(fMsg.getGetTsvDirE001(result.getAbsolutePath()));
		}
		if (result.isDirectory() == false) {
			throw new IllegalArgumentException(fMsg.getGetTsvDirE002(result.getAbsolutePath()));
		}

		return result;
	}

	/**
	 * 拡張ポイントから用語集データ TSV プラグインを取得します。
	 * @return 用語集データ TSV プラグイン。必ず null 以外が戻ります。
	 */
	private AbstractBentenGlossaryDataTsvPlugin getTsvDataPlugin() {
		final IExtensionRegistry registry = Platform.getExtensionRegistry();
		final IExtensionPoint point = registry.getExtensionPoint("benten.cat.glossary.data.tsv"); //$NON-NLS-1$

		if (point == null) {
			throw new IllegalArgumentException(fMsg.getGetTsvDataPluginE001());
		}

		final Map<String, AbstractBentenGlossaryDataTsvPlugin> tsvMap = new HashMap<String, AbstractBentenGlossaryDataTsvPlugin>();

		for (IExtension extension : point.getExtensions()) {
			for (IConfigurationElement cfgElem : extension.getConfigurationElements()) {
				try {
					final String name = cfgElem.getAttribute("name"); //$NON-NLS-1$
					final Object tsvPluginClass = cfgElem.createExecutableExtension("class"); //$NON-NLS-1$
					if (tsvPluginClass instanceof Plugin) {
						tsvMap.put(name, (AbstractBentenGlossaryDataTsvPlugin) tsvPluginClass);
					}
				} catch (final CoreException e) {
					getDefault().log(e);
				}
			}
		}

		final AbstractBentenGlossaryDataTsvPlugin tsvPlugin = tsvMap.get(DEFAULT_GLOSSARY);
		if (tsvPlugin == null) {
			throw new IllegalArgumentException(fMsg.getGetTsvDataPluginE002());
		}

		return tsvPlugin;
	}
}
